local M = {}
local constants = {
    -- Conversion rates
    avToRPM = 9.549296596425384,
    rpmToAV = 1.0 / 9.549296596425384,
}

local function getExtraData(engine)
    local extraData = engine["__engineUtilExtraData"]
    
    if not extraData then
        extraData = {}
        engine["__engineUtilExtraData"] = extraData
    end
    
    return extraData
end

local function getTorqueData(engine, forceFresh)
    local data = getExtraData(engine)
    
    if forceFresh or not data.torqueData then
        data.torqueData = engine:getTorqueData()
    end
    
    return data.torqueData
end

local function getBestRpmForTorque(engine, torque)
    local torqueData = getTorqueData(engine)
    local curveData = torqueData.curves[torqueData.finalCurveName]
    local torqueCurve = curveData.torque
    
    for i = engine.idleRPM, torqueData.maxTorqueRPM, 10 do
        if torqueCurve[i] >= torque then
            return i
        end
    end
    
    return torqueData.maxTorqueRPM
end

local function getAchievableEngineTorque(engine, forceFresh)
    local torqueData = getTorqueData(engine, forceFresh)
    local curveData = torqueData.curves[torqueData.finalCurveName]
    local torqueCurve = curveData.torque
    local rpm = math.floor(engine.outputAV1 * constants.avToRPM)
    local rawMaxTorque = torqueCurve[rpm] or 0
    
    return math.abs(rawMaxTorque - engine.friction - (engine.dynamicFriction * engine.outputAV1))
end

local function getInstantMaxEngineTorque(engine)
    local achievableEngineTorque = getAchievableEngineTorque(engine)
    
    return math.abs(achievableEngineTorque * engine.forcedInductionCoef)
end

M.getBestRpmForTorque = getBestRpmForTorque
M.getAchievableEngineTorque = getAchievableEngineTorque
M.getInstantMaxEngineTorque = getInstantMaxEngineTorque

return M