local M = {}

-- Timed Latch
local timedLatch = {}
timedLatch.__index = timedLatch

local function newTimedLatch(timeToLatch, immediateLatchOnTrue, immediateLatchOnFalse)
    local obj = {
        timeToLatch = timeToLatch,
        value = false,
        elapsedTime = 0,
        immediateTrue = immediateLatchOnTrue == true,
        immediateFalse = immediateLatchOnFalse == true,
    }
    
    setmetatable(obj, timedLatch)
    return obj
end

function timedLatch:test(value, dt)
    if value == true and self.immediateTrue then
        self.elapsedTime = 0
        self.value = true
        return true
    end
    
    if value == false and self.immediateFalse then
        self.elapsedTime = 0
        self.value = false
        return false
    end
    
    self.elapsedTime = self.elapsedTime + dt
    
    if self.elapsedTime >= self.timeToLatch then
        self.elapsedTime = 0
        self.value = value
        
        return value
    else
        return self.value
    end
end

function timedLatch:reset(value)
    value = not not value -- coerce to boolean
    
    self.elapsedTime = 0
    self.value = value
end

M.newTimedLatch = newTimedLatch

-- Threshold latch
local thresholdLatch = {}
thresholdLatch.__index = thresholdLatch

local function newThresholdLatch(thresholdOn, thresholdOff, initialState)
    local obj = {
        tOn = thresholdOn,
        tOff = thresholdOff,
        state = initialState or false,
    }
    
    setmetatable(obj, thresholdLatch)
    return obj
end

function thresholdLatch:test(value)
    if self.state then
        self.state = value >= self.tOff
    else
        self.state = value >= self.tOn
    end
    
    return self.state
end

M.newThresholdLatch = newThresholdLatch

return M